XXX Not complete !!!

Name

    SGIS_texture_filter4

Name Strings

    GL_SGIS_texture_filter4

Version

    $Date: 1995/09/30 02:33:15 $ $Revision: 1.1 $

Number

    7

Dependencies

    None

Overview

    This extension allows 1D and 2D textures to be filtered using an
    application-defined, four sample per dimension filter.  (In addition to
    the NEAREST and LINEAR filters defined in the original GL Specification.)
    Such filtering results in higher image quality.  It is defined only
    for non-mipmapped filters.  The filter that is specified must be
    symmetric and separable (in the 2D case).

Issues

    *	What should the default filter function be?

    *	Should this extension define 2-wide texture borders?  Do we really
	want to aggrandize this border stuff :-(

    *	Should this extension define 2D filtering only (and not 1D)?

    *	A GLU function that accepts Mitchell parameters and a texture
	target should be defined.

    *	This specification retains a separate filter function description
	with every texture.  In conjunction with EXT_texture_object, this
	may result in a lot of filter functions.  Implementations should
	optimize for the default filter function to save storage cost.

Reasoning

    *	The name is changed from "cubic" to "filter4" because the table
	allows the specification of filters that are not at all cubic in
	nature.  A true cubic filter extension would define the filter
	function as a cubic polynomial.

    *	WeightGranularity and implementation rounding of filter weights are
	specified so that applications can be sure to define filter
	functions that sum to 1.0 at all sample offsets.

    *	I have packed the seven values that describe the implementation
	specific filter4 table parameters into a single query, rather
	than providing separate queries in the GL style.  Since any
	application that needs one of these parameters needs all of them,
	it seems unacceptable to force multiple round trips, especially
	across a wire.

New Procedures and Functions

    void TexFilterFuncSGIS(enum target,
			   enum filter,
			   const float* weights);

    void GetTexFilterFuncSGIS(enum target,
			      enum filter, 
			      float* weights);

New Tokens

    Accepted by the <param> parameter of TexParameteri and TexParameterf,
    and by the <params> parameter of TexParameteriv and TexParameterfv, when
    their <pname> parameter is TEXTURE_MIN_FILTER or TEXTURE_MAG_FILTER.
    Also accepted by the <filter> parameters of TexFilterFuncSGIS and
    GetTexFilterFuncSGIS:

	FILTER4_SGIS

    Accepted by the <pname> parameter of GetTexParameteriv and
    GetTexParameterfv, when the <target> parameter is TEXTURE_1D or
    TEXTURE_2D:

	TEXTURE_FILTER4_PARAMETERS_SGIS

Additions to Chapter 2 of the GL Specification (OpenGL Operation)

    None

Additions to Chapter 3 of the GL Specification (Rasterization)

    The additional token value FILTER4_SGIS is accepted as an enumerated
    value for the texture minification and magnification filters, causing
    Table 3.7 to be replaced with the table below:

	Name				Type		Legal Values
	----				----		------------
	TEXTURE_WRAP_S			integer		CLAMP, REPEAT
	TEXTURE_WRAP_T			integer		CLAMP, REPEAT
	TEXTURE_WRAP_R_EXT		integer		CLAMP, REPEAT
	TEXTURE_MIN_FILTER		integer		NEAREST, LINEAR,
							NEAREST_MIPMAP_NEAREST,
							NEAREST_MIPMAP_LINEAR,
							LINEAR_MIPMAP_NEAREST,
							LINEAR_MIPMAP_LINEAR,
							FILTER4_SGIS
	TEXTURE_MAG_FILTER		integer		NEAREST, LINEAR,
							FILTER4_SGIS
	TEXTURE_BORDER_COLOR		4 floats	any 4 values in [0,1]

	Table 3.7: Texture parameters and their values.

    Filter4 filtering is specified by calling TexParameteri, TexParameterf,
    TexParameteriv, or TexParameterfv with <pname> set to one of
    TEXTURE_MIN_FILTER or TEXTURE_MAG_FILTER, and <param> or <params> set to
    FILTER4_SGIS.  Because filter4 filtering is defined only for
    non-mipmapped textures, there is no difference between its definition
    for minification and magnification.  First consider the 1-dimensional
    case.  Let T be a computed texture value (one of R_t, G_t, B_t, or A_t).
    Let T[i] be the component value of the texel at location i in a
    1-dimensional texture image.  Then, if the appropriate texture filter
    mode is FILTER4_SGIS, a 4-texel group is selected:


	      /	floor(u - 1/2) mod 2**n,	TEXTURE_WRAP_S is REPEAT
	i1 = (
	      \	floor(u - 1/2),			TEXTURE_WRAP_S is CLAMP


	      /	(i1 + 1) mod 2**n,		TEXTURE_WRAP_S is REPEAT
	i2 = (
	      \	i1 + 1,				TEXTURE_WRAP_S is CLAMP


	      /	(i1 + 2) mod 2**n,		TEXTURE_WRAP_S is REPEAT
	i3 = (
	      \	i1 + 2,				TEXTURE_WRAP_S is CLAMP


	      /	(i1 - 1) mod 2**n,		TEXTURE_WRAP_S is REPEAT
	i0 = (
	      \	i1 - 1,				TEXTURE_WRAP_S is CLAMP


    Let f(x) be the filter weight function of positive distance x.  Let
    
	A = frac(u - 1/2)

    where frac(x) denotes the fractional part of x, and u is the texture
    image coordinate in the s direction, as illustrated in Figure 3.10 of
    the GL Specification.  Then the texture value T is found as

	T = f(1+A) * T[i0] +
	    f(A)   * T[i1] + 
	    f(1-A) * T[i2] +
	    f(2-A) * T[i3]

    If any of the selected T[i] in the above equation refer to a border
    texel with unspecified value, then the border color given by the current
    setting of TEXTURE_BORDER_COLOR is used instead of the unspecified value.

    For 2-dimensional textures the calculations for i0, i1, i2, i3, and A
    are identical to the 1-dimensional case.  A 16-texel group is selected,
    requiring four j values computed as


	      /	floor(v - 1/2) mod 2**m,	TEXTURE_WRAP_T is REPEAT
	j1 = (
	      \	floor(v - 1/2),			TEXTURE_WRAP_T is CLAMP


	      /	(j1 + 1) mod 2**m,		TEXTURE_WRAP_T is REPEAT
	j2 = (
	      \	j1 + 1,				TEXTURE_WRAP_T is CLAMP


	      /	(j1 + 2) mod 2**m,		TEXTURE_WRAP_T is REPEAT
	j3 = (
	      \	j1 + 2,				TEXTURE_WRAP_T is CLAMP


	      /	(j1 - 1) mod 2**m,		TEXTURE_WRAP_T is REPEAT
	j0 = (
	      \	j1 - 1,				TEXTURE_WRAP_T is CLAMP


    Let

	B = frac(v - 1/2)

    where v is the texture image coordinate in the t direction, as
    illustrated in Figure 3.10 of the GL Specification.  Then the texture
    value T is found as

	T = f(1+A) * f(1+B) * T[i0,j0] +
	    f(A)   * f(1+B) * T[i1,j0] + 
	    f(1-A) * f(1+B) * T[i2,j0] +
	    f(2-A) * f(1+B) * T[i3,j0] +
	    f(1+A) * f(B)   * T[i0,j1] +
	    f(A)   * f(B)   * T[i1,j1] + 
	    f(1-A) * f(B)   * T[i2,j1] +
	    f(2-A) * f(B)   * T[i3,j1] +
	    f(1+A) * f(1-B) * T[i0,j2] +
	    f(A)   * f(1-B) * T[i1,j2] + 
	    f(1-A) * f(1-B) * T[i2,j2] +
	    f(2-A) * f(1-B) * T[i3,j2] +
	    f(1+A) * f(2-B) * T[i0,j3] +
	    f(A)   * f(2-B) * T[i1,j3] + 
	    f(1-A) * f(2-B) * T[i2,j3] +
	    f(2-A) * f(2-B) * T[i3,j3]

    If any of the selected T[i,j] in the above equation refer to a border
    texel with unspecified value, then the border color given by the current
    setting of TEXTURE_BORDER_COLOR is used instead of the unspecified value.

    Filter4 texture filtering is currently undefined for 3-dimensional
    textures.

    Filter function
    ---------------

    By default the filter function is XXX.  The filter function is
    specified in table format by calling TexFilterFuncSGIS with <target>
    set to TEXTURE_1D or TEXTURE_2D, <filter> set to FILTER4_SGIS, and
    <weights> pointing an array of N floating point values.  Each value
    is first clamped to the range [MinWeight, MaxWeight], then rounded by
    the implementation to the nearest discrete value V, such that

	V = MinWeight + i * WeightGranularity

    for some integer value i.  The first weight defines the filter
    function f(Min).  Subsequent weight values j, where j is 1 through
    N-1, describe filter functions f(Min+j*Granularity).

    The values Min, Max, Granularity, N, MinWeight, MaxWeight, and 
    WeightGranularity for texture <target> are returned by <params> in that
    order when GetTexParameteriv or GetTexParameterfv is called with <pname>
    set to TEXTURE_FILTER4_PARAMETERS_SGIS.  Max is the distance of the last
    filter weight, and satisfies the equation

	Max = Min + (N-1) * Granularity

    to within the precision of the floating point arithmetic.

    Minimum and maximum values for the implementation dependent constants
    are defined in the table below:

	Constant Name		Constraint
	-------------		----------
	Min			<= 0.0625
	Max			>= 1.875
	Granularity		<= 0.125
	N			>= 16
	MinWeight		<= -0.5
	MaxWeight		>=  1.0
	WeightGranularity	<= 0.125

    Minification vs. Magnification
    ------------------------------

    If the magnification filter is given by FILTER4_SGIS, and the
    minification filter is given by NEAREST_MIPMAP_NEAREST or
    LINEAR_MIPMAP_NEAREST , then c = 0.5.  The parameter c is used to
    determine whether minification or magnification filtering is done,
    as described in Section 3.8.2 of the GL Specification (Texture
    Magnification).

Additions to Chapter 4 of the GL Specification (Per-Fragment Operations
and the Framebuffer)

    None

Additions to Chapter 5 of the GL Specification (Special Functions)

    GetTexFilterFuncSGIS is not included in display lists.

Additions to Chapter 6 of the GL Specification (State and State Requests)

    The filter weights for the filter4 filter associated with a texture
    are queried by calling GetTexFilterFuncSGIS with <target> set to the
    texture target and <filter> set to FILTER4_SGIS.  The N weight values
    are returned in the array <weights>, which must have at least N
    elements.  The value N is a constant that s queried by the application
    by calling GetTexParameteriv or GetTexParameterfv as described above.

Additions to the GLX Specification

    None

Errors

    INVALID_ENUM is generated if TexFilterFuncSGIS or
    GetTexFilterFuncSGIS parameter <target> is not TEXTURE_1D or
    TEXTURE_2D.

    INVALID_ENUM is generated if TexFilterFuncSGIS or
    GetTexFilterFuncSGIS parameter <filter> is not FILTER4_SGIS.

    INVALID_ENUM is generated if GetTexParameteriv or GetTexParameterfv
    parameter <pname> is TEXTURE_FILTER4_PARAMETERS_SGIS and parameter
    <target> is not TEXTURE_1D or TEXTURE_2D.

    INVALID_OPERATION is generated if TexFilterFuncSGIS or
    GetTexFilterFuncSGIS is executed between execution of Begin and the
    corresponding execution of End.

New State

									Initial
    Get Value			Get Command		Type		Value		Attrib
    ---------			-----------		----		-------		------
    TEXTURE_FILTER4_WEIGHTS	GetTexFilterFuncSGIS	2 x N x R	see text	texture

New Implementation Dependent State

										Minimum
    Get Value				Get Command		Type		Value
    ---------				-----------		----		-------

    TEXTURE_FILTER4_PARAMETERS_SGIS	GetTexParameterfv	2 x 7 x R	see text
