Name

    EXT_visual_rating

Name Strings

    GLX_EXT_visual_rating

Version

    $Date: 1995/09/30 02:33:57 $ $Revision: 1.1 $

Number

    44

Dependencies

    None

Overview

    This extension allows servers to identify each GLX visual as belonging
    to one of three categories: good, bad, and ugly.  A visual's category,
    called its rating, is queried using glXGetConfig, and can therefore be
    used by application-specific visual selection routines.  The rating is
    also used by glXChooseVisual as a part of its selection criteria.
    Good visuals are treated as they would be if the extension were not
    present.  Bad visuals are returned only if no good visual satisfies the
    specified requirements.  Ugly visuals are returned only if the token
    value GLX_UGLY_VISUAL_OK_EXT is included in <attrib_list>, and then only
    if no good or bad visual satisfies the other specified requirements.

    This extension allows servers to export visuals with improved features
    or image quality, but lower performance or greater system burden,
    without having to have these visuals selected preferentially.  It is
    intended to insure that most applications get the "right" visual, not
    that all applications do.

Issues

    Can we get away with naming the ratings good, bad, and ugly? :-)

    This extension changes the behavior of glXChooseVisual, and will
    therefore affect the operation of existing GL applications.

Reasoning

    This extension is designed to be automatic -- applications don't have
    to use it explicitly.  The only explicit action an application can take
    is to allow glXChooseVisual to return ugly visuals.  glXChooseVisual
    always prefers good visuals over bad ones, and by default will not
    return an ugly visual.

    Because unrated visuals are treated as good visuals, applications need
    not concern themselves with whether a server supports EXT_visual_rating
    or not.  This allows operation to be automatic.

New Procedures and Functions

    None

New Tokens

    Accepted by the <attribute> parameter of glXGetConfig:

	GLX_VISUAL_RATING_EXT		0x20

    Accepted by the <attrib_list> parameter of glXChooseVisual:

	GLX_UGLY_VISUAL_OK_EXT		0x21

    Returned by the <value> parameter of glXGetConfig:

	GLX_GOOD_VISUAL_EXT		0x1
	GLX_BAD_VISUAL_EXT		0x2
	GLX_UGLY_VISUAL_EXT		0x3

Additions to Chapter 2 of the 1.0 Specification (OpenGL Operation)

    None

Additions to Chapter 3 of the 1.0 Specification (Rasterization)

    None

Additions to Chapter 4 of the 1.0 Specification (Per-Fragment Operations
and the Frame Buffer)

    None

Additions to Chapter 5 of the 1.0 Specification (Special Functions)

    None

Additions to Chapter 6 of the 1.0 Specification (State and State Requests)

    None

Additions to the GLX Specification

    When called with <attribute> set to GLX_VISUAL_RATING_EXT, glXGetConfig
    returns in parameter <value> one of three values: GLX_GOOD_VISUAL_EXT,
    GLX_BAD_VISUAL_EXT, or GLX_UGLY_VISUAL_EXT.  The returned value is
    known as the rating of the visual.

    glXChooseVisual returns a pointer to an XVisualInfo structure describing
    the visual with rating GLX_GOOD_VISUAL_EXT that best meets a minimum
    specification.  If no visual with rating GLX_GOOD_VISUAL_EXT meets the
    minimum specification, glXChooseVisual returns a pointer to an
    XVisualInfo structure describing the visual with rating
    GLX_BAD_VISUAL_EXT that best meets a minimum specification.  If no
    visual with rating GLX_GOOD_VISUAL_EXT or GLX_BAD_VISUAL_EXT meets the
    minimum specification, and if <attrib_list> includes the parameter
    GLX_UGLY_VISUAL_OK_EXT, glXChooseVisual returns a pointer to an
    XVisualInfo structure describing the visual with rating
    GLX_UGLY_VISUAL_EXT that best meets the minimum specification.
    Otherwise, glXChooseVisual returns NULL.

    If the server doesn't support EXT_visual_rating, GLX client
    implementations treat all visuals as having rating GLX_GOOD_VISUAL_EXT.
    Individual visuals that are not rated are also treated as good visuals.
    A client that supports EXT_visual_rating will insure that the string
    GLX_EXT_visual_rating is returned by glXQueryExtensionsString,
    regardless of whether the server supports EXT_visual_rating.

GLX Protocol

    A single additional property type/property value pair may be included in
    the property list of each visual returned by glXGetVisualConfigs.  This
    property type/property value pair is encoded as

     4         ENUM            property type
               0x20            GLX_VISUAL_RATING_EXT
     4         ENUM            property value
               0x1             GLX_GOOD_VISUAL_EXT
               0x2             GLX_BAD_VISUAL_EXT
               0x3             GLX_UGLY_VISUAL_EXT


    If this property type/property value pair is not specified for a visual,
    that visual is understood to have rating GLX_GOOD_VISUAL_EXT.


Errors

    None

New State

    No GL state

New Implementation Dependent State

    None
